#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <exBasics.h>
#include <exEdit.h>
#include <exEntry.h>
#include <exLayout.h>
#include <exErrors.h>
#include <exMenu.h>

#include "version.h"
#include "aedinfo.h"

/****************************************************************************
 * constant definitions 
 ****************************************************************************/

#define FMMENUS     "fm.menu"		/* name of the menudescr file */
#define FGREP       "/usr/bin/fgrep"	/* absolute path of fgrep command */

#define DEF_XPOS	460
#define DEF_YPOS        0

#define BORDERWIDTH        2
#define SHADOW_THICKNESS   3 		/* shadow-thickness of tree-buttons */
#define SEP_THICKNESS      4
  
#define UPDATE_INTERVAL    60		/* interval between update checks */

/* for tree-representation */
#define MAXROW                  100
#define MAXCOL                  20
#define MAXNODES                500
#define NODELEN                 12   /* 10 */
#define DIRBUTTONWIDTH(f)    (exFontWidth(f)* NODELEN + 2 * SHADOW_THICKNESS)
#define DIRBUTTONHEIGHT(f)   (exFontHeight(f) + 2 * SHADOW_THICKNESS)
#define DIRBUTTONHGAP           20
#define DIRBUTTONVGAP           5
#define DEFNODESPERLINE         4	/* default number of nodes per line */
#define DEFNODESPERROW          10	/* default number of nodes per row */

#define NODE_SELECTED    	0x1	/* the nod is selected */
#define NODE_DISPLAYED          0x2     /* the directory is displayed */

#define ITEMSPERLINE 	   3		/* items per line */
#define LINESPERWINDOW    10		/* lines per window */
#define ITEMLEN           15            /* length of item name */
#define ITEMFLAGLEN        6            /* additional space for item flags */
#define MAXITEMLEN        (ITEMLEN + ITEMFLAGLEN)
#define PATHLEN		 512

#define ITEM_VISIBLE    0x1	/* item will be displayed */
#define ITEM_SELECTED   0x2     /* item has been selected */
#define ITEM_DELETED    0x4     /* file will be deleted   */
#define ITEM_ISDIR      0x8     /* item denotes a directory */
#define ITEM_RENAMED    0x10    /* item has been renamed */
#define ITEM_DCHGD      0x20    /* file description changed (chmod,... ) */
#define ITEM_DISPLAYED  0x40    /* the items is a directory and is displayed */
#define ITEM_ERROR      0x80    /* error while reading item-information */

#define DIRW_VERT       0x1     /* items are displayed vertically ordered */
#define DIRW_HOR        0x2     /* items are displayed vertically ordered */
#define DIRW_CHNGED     0x4     /* directory has been changed by user */
#define DIRW_FIRST	0x8	/* dir window is displayed the first time */

#define ORDER_NOORDER   0       /* items are not ordered */
#define ORDER_BYNAME    1 	/* items are ordered by name */
#define ORDER_BYACCESS  2 	/* items are ordered by accesstime */
#define ORDER_BYMOD     3 	/* ordered by modificationtime */
#define ORDER_BYSIZE    4    	/* order by size of file */

/* selection parameter */
#define SELECT_ALL      0	/* select all files */
#define UNSELECT_ALL    1	/* deselect all files */
#define SELECT_BY_GREP  2	/* select files that contain pattern */
#define ALL_WINDOWS     3	/* for all windows */
#define CURRENT_WINDOW  4       /* only for the current window */

/* abort or quit graceful */
#define FMABORT        0
#define FMQUIT         1

/* list directory parameter */
#define DIR_FATHER   1		/* list father directory */
#define DIR_ANY      2		/* list directory that is entered by user */
#define DIR_SELECTED 3		/* list currently selected directory */

/* tree window parameters */
#define VSCROLL		0x1	/* vertical scrollbar is active */
#define HSCROLL		0x2	/* horizontal scrollbar is active */


/****************************************************************************
 * makro definitions 
 ****************************************************************************/

#define ITEM_NAME(iptr) (((iptr)->flags & ITEM_RENAMED) \
			 	? (iptr)->a_name : (iptr)->D_name)

#define SET_ITEM_STATE(wnd, iptr) if ((iptr)->flags & ITEM_VISIBLE) {       \
				    if ((wnd)->m->selections[(iptr)->slot]) \
				      (iptr)->flags |= ITEM_SELECTED;       \
   			            else				    \
				      (iptr)->flags &= ~ITEM_SELECTED;      \
				  }

#define SET_MENU_STATE(wnd, iptr)  if ((iptr)->slot >= 0)		    \
  				   (wnd)->m->selections[(iptr)->slot] =     \
  				    ((iptr)->flags & ITEM_SELECTED) ? 1 : 0;

/****************************************************************************
 * structure definitions 
 ****************************************************************************/
 
typedef struct _dirWndHead {
  	GC fm_gc;			/* graphics context */
	int curx, cury;			/* coord. of next dir. window */
	ApplicationNode *fm_aptr;	/* application node */
  	Widget root;			/* the root widget */
        Editor attrEditor;		/* attribute editor */
	Widget attrShell,		/* editor top-level shell */
	       attrWindow,		/* editor main-window */
	       attrTitle,		/* editor title window */
	       attrCloseBox;		/* close box of attr editor */
        struct _dirWindow *first;	/* first dir window */
	struct _dirWindow *last;        /* last in chain */
	aedForm *aed_form;	       	/* attribute editor form */
	XFontStruct *itemFont;		/* font for items in dir. window */
	Dimension title_height;		/* height of title */
	Dimension mb_height;		/* height of menu bar */
	Dimension sb_width;
} dirWndHead;

typedef struct _dirWindow {
	struct _dirWindow *next, *prev; /* chaining windows */
	Widget fmTop; 			/* the whole Widget */
	Widget fmWindow;		/* top-most form window */
	Widget title;			/* title widget */
	Widget closeBox;		/* push-button to iconise */
	Widget workArea; 		/* the Widget containing items */
	struct {			/* icon related information */
	  Widget iconShell;
	  Widget icon;
	  Widget iconTitle;
	  Widget iconWindow;
 	  Widget iconCloseBox;
	} dirwIcon;
	exMenubar *menubar;		/* the menubar */
	exMenu *m;			/* associated menu structure */
	struct _dirItem *itemList;      /* the list of items */
	struct _treeWindow *tree;	/* pointer to tree-structure widget */
	char *path,                     /* directory name as displayed */
	     *u_path;    		/* absolute UNIX - path */
	int itemCnt,			/* # of read items */
	    w_lines,			/* # of lines displayed */
	    w_cols;			/* # of items per line */
	time_t readTime; 		/* modification-time of the directory 
					   when beeing read */
	int flags;                      /* define several properties */
	int sortFlag;               	/* how to sort items */
	char *filePattern;	        /* pattern to get visible files */
} dirWindow;

typedef struct _dirItem {
  	int slot;		        /* no. of correspoding menu item */
	struct _dirItem *next;		/* chaining items */
	struct stat *sbp;		/* stat information */
	char *D_name;			/* original name */
	char *a_name;                   /* new name, if renamed */
	int flags;			/* additonal flags */
	struct _dirWindow *dirWnd;	/* the associated directory window */
} dirItem;

typedef struct _treeWindow {
  Widget shell;				/* toplevel shell */
  Widget title;				/* title widget */
  Widget w;				/* the visible window */
  Widget work;				/* the visible part of the tree */
  Widget plane;				/* the "large" back-window */
  Widget closeBox;			/* closebox button */
  Widget h_scrollb;
  Widget v_scrollb;
  Widget separator;
  XFontStruct *font;			/* button font */
  dirWindow *wnd;			/* associated directory window */
  XSegment connections[MAXNODES]; 	/* lines between nodes */
  int conCnt;				/* number of connections */
  int cols, rows;			/* number of rows and cols */
  int cur_row, cur_col;			/* col and row of top-left node */
  char *path;				/* unix pathname */
  struct _dirNode *nodes;		/* the list of nodes */
  int flags;
} treeWindow;

typedef struct _dirNode {
  int x, y;				/* upper-left corner of button */
  char *name;				/* full qualified file name */
  int flags;
  Widget node;				/* the button window */
  struct _dirNode *next;		/* next node in list */
} dirNode;

static char fmSccsID[] = "%W%\t%G%";            /* sccs keyword */

/*****************************************************************************
 * external declarations 
 ****************************************************************************/

extern foreground, background;
extern Pixmap iconPixmap, closeBoxPixmap;
extern treeWindow *curTreeWnd;
extern dirWndHead *Head;
extern Cursor MoveCursor, PanelCursor, CopyCursor, TreeCursor;
extern unsigned long fm_white_pixel, fm_black_pixel;
/* external defintions */



